'use strict';
const axios = require('axios');

//PROD Environment
const BLOCK_API_URL_BASE = 'https://builder.developer.riiiver.com/api/';

const getAccessToken = async (userData, serviceProxy) => {
  const startTime = new Date();
  console.log('OAuth2Util: Get access_token');
  try {
    console.log(`OAuth2Util: serviceProxy=${serviceProxy}`);
    const blockId = await getBlockId(userData, serviceProxy);
    if (!blockId) {
      throw new Error('OAuth2Util error: Block not found');
    }
    return await getSavedAccessToken(userData, blockId);
  } catch (e) {
    console.log(e);
    throw e;
  } finally {
    console.log(
      `OAuth2Util getAccessToken complete in 
       ${new Date() - startTime}
       ms`
    );
  }
};

function getBlockId(userData, serviceProxy) {
  const options = {
    url: `${BLOCK_API_URL_BASE}proxyBlockId`,
    method: 'get',
    headers: {
      'User-Agent': userData.userAgent,
      'Authorization': userData.credential
    },
    params: {
      // `params` are the URL parameters to be sent with the request
      serviceProxy: serviceProxy
    },
  };
  return axios.request(options)
    .then(function (response) {
      console.log('OAuth2Util getBlockId: ', JSON.stringify(response.data));
      return response.data.blockId;
    }).catch(function (e) {
      console.error('OAuth2Util error: unable to get blockId');
      return null;
    });
};

function getSavedAccessToken(userData, blockId) {
  console.log('OAuth2Util getSavedAccessToken: start');
  const options = {
    url: `${BLOCK_API_URL_BASE}clientAuthInfo`,
    method: 'get',
    headers: {
      'User-Agent': userData.userAgent,
      'Authorization': userData.credential
    },
    params: {
      blockId: blockId,
      uuid: userData.uuid
    },
  };
  return axios.request(options)
    .then(function (response) {
      console.log(JSON.stringify(response.data));
      const body = response.data;
      const expiresIn = body.oauth2.expires_in;
      let accessToken;
      const now = Math.floor(new Date().getTime() / 1000);
      if (expiresIn <= now - 60 * 5) {
        accessToken = refreshAccessToken(userData, blockId, body.oauth2.refresh_token);
      } else {
        accessToken = body.oauth2.access_token;
      }
      console.log(`OAuth2Util getSavedAccessToken: complete, accessToken = ${accessToken}`);
      return accessToken;
    }).catch(function (e) {
      console.error('OAuth2Util error: unable to get accessToken', JSON.stringify(e));
      return null;
    })
};

function refreshAccessToken(userData, blockId, refresh_token) {
  console.log('OAuth2Util refreshAccessToken: start');
  const options = {
    url: `${BLOCK_API_URL_BASE}updateOAuthToken`,
    method: 'put',
    headers: {
      'User-Agent': userData.userAgent,
      'Authorization': userData.credential,
      'Content-Type': 'application/json'
    },
    params: {
      'blockId': blockId,
      'uuid': userData.uuid
    },
    data: {
      'refresh_token': refresh_token
    }
  };
  return axios.request(options)
    .then(function (response) {
      console.log('OAuth2Util refreshAccessToken: complete', response.data);
      return response.data.access_token;
    }).catch(function (e) {
      console.log('OAuth2Util error: unable to refesh accessToken', JSON.stringify(e));
      return null;
    })
};

module.exports = {
  getAccessToken
};
